#define CONFIG_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/XmAll.h>
#include <stdio.h>
#include <unistd.h>
#include <fnmatch.h>
#include <values.h>

#include "interface.h"
#include "pixmap.h"
#include "error_handler.h"

/* File Format structure */

typedef struct {
  char *description;
  char *pattern;
  char *view_command;
  char *edit_command;
  char *pixmap_name;
  Pixmap pixmap;
} TFormatList;

/* Variable structure */

typedef struct {
  char *name;
  int int_val;
  Bool bool_val;
  char *str_val;
} TVarList;

/* User Menu structure */

typedef enum { POPUP, MENUITEM, SEPARATOR } TMenuType;

typedef struct {
  TMenuType type;
  char *name;
  char *command;
} TMenuEntry;

typedef struct {
  char *ident;
  char *name;
  int nb_entries;
  TMenuEntry *entry;
} TPopupList;

/* global vars */

TFormatList *G_FormatList=NULL;
int G_NbFormats=0;
TVarList *G_VarList=NULL;
int G_NbVars=0;
TPopupList *G_MenuList=NULL;
int G_NbMenus=0;
TPopupList *G_CMenuList=NULL;
int G_NbCMenus=0;
char **G_PixmapPath=NULL;
int G_NbPaths=0;

/*--------------------------------------------------------------------------*/
/* user menu functions   */
/*-----------------------*/

void AddMenu(char *ident, char *name)
{
  G_NbMenus++;
  G_MenuList=(TPopupList *) XtRealloc((char *)G_MenuList, sizeof(TPopupList)*G_NbMenus);
  G_MenuList[G_NbMenus-1].ident=ident;
  G_MenuList[G_NbMenus-1].name=name;
  G_MenuList[G_NbMenus-1].nb_entries=0;
  G_MenuList[G_NbMenus-1].entry=NULL;
}

void AddMenuItem(char *name, char *command)
{
  G_MenuList[G_NbMenus-1].nb_entries++;
  G_MenuList[G_NbMenus-1].entry=(TMenuEntry *) XtRealloc((char *)G_MenuList[G_NbMenus-1].entry, sizeof(TMenuEntry)*G_MenuList[G_NbMenus-1].nb_entries);
  G_MenuList[G_NbMenus-1].entry[G_MenuList[G_NbMenus-1].nb_entries-1].name=name;
  G_MenuList[G_NbMenus-1].entry[G_MenuList[G_NbMenus-1].nb_entries-1].command=command;
  G_MenuList[G_NbMenus-1].entry[G_MenuList[G_NbMenus-1].nb_entries-1].type=MENUITEM;
}

void AddMenuSeparator(void)
{
  G_MenuList[G_NbMenus-1].nb_entries++;
  G_MenuList[G_NbMenus-1].entry=(TMenuEntry *) XtRealloc((char *)G_MenuList[G_NbMenus-1].entry, sizeof(TMenuEntry)*G_MenuList[G_NbMenus-1].nb_entries);
  G_MenuList[G_NbMenus-1].entry[G_MenuList[G_NbMenus-1].nb_entries-1].name=NULL;
  G_MenuList[G_NbMenus-1].entry[G_MenuList[G_NbMenus-1].nb_entries-1].command=NULL;
  G_MenuList[G_NbMenus-1].entry[G_MenuList[G_NbMenus-1].nb_entries-1].type=SEPARATOR;
}

void AddMenuPopup(char *ident)
{
  G_MenuList[G_NbMenus-1].nb_entries++;
  G_MenuList[G_NbMenus-1].entry=(TMenuEntry *) XtRealloc((char *)G_MenuList[G_NbMenus-1].entry, sizeof(TMenuEntry)*G_MenuList[G_NbMenus-1].nb_entries);
  G_MenuList[G_NbMenus-1].entry[G_MenuList[G_NbMenus-1].nb_entries-1].name=ident;
  G_MenuList[G_NbMenus-1].entry[G_MenuList[G_NbMenus-1].nb_entries-1].command=NULL;
  G_MenuList[G_NbMenus-1].entry[G_MenuList[G_NbMenus-1].nb_entries-1].type=POPUP;
}

void DestroyPopupList(void)
{
  int i,j;
  
  for(i=0;i<G_NbMenus;i++) {
    for(j=0;j<G_MenuList[i].nb_entries;j++) 
      switch(G_MenuList[i].entry[j].type) {
      case MENUITEM :
	XtFree(G_MenuList[i].entry[j].name);
	XtFree(G_MenuList[i].entry[j].command);
	break;
      case POPUP :
	XtFree(G_MenuList[i].entry[j].name);
	break;
      default:
      }
    XtFree((char*) G_MenuList[i].entry);
    XtFree(G_MenuList[i].ident);
    XtFree(G_MenuList[i].name);
  }
  XtFree((char *)G_MenuList);
  G_MenuList=NULL;
  G_NbMenus=0;
}
  
/*--------------------------------------------------------------------------*/
/* context menus functions   */
/*-----------------------*/

void AddCMenu(char *ident, char *name)
{
  G_NbCMenus++;
  G_CMenuList=(TPopupList *) XtRealloc((char *)G_CMenuList, sizeof(TPopupList)*G_NbCMenus);
  G_CMenuList[G_NbCMenus-1].ident=ident;
  G_CMenuList[G_NbCMenus-1].name=name;
  G_CMenuList[G_NbCMenus-1].nb_entries=0;
  G_CMenuList[G_NbCMenus-1].entry=NULL;
}

void AddCMenuItem(char *name, char *command)
{
  G_CMenuList[G_NbCMenus-1].nb_entries++;
  G_CMenuList[G_NbCMenus-1].entry=(TMenuEntry *) XtRealloc((char *)G_CMenuList[G_NbCMenus-1].entry, sizeof(TMenuEntry)*G_CMenuList[G_NbCMenus-1].nb_entries);

  G_CMenuList[G_NbCMenus-1].entry[G_CMenuList[G_NbCMenus-1].nb_entries-1].name=name;
  G_CMenuList[G_NbCMenus-1].entry[G_CMenuList[G_NbCMenus-1].nb_entries-1].command=command;
  G_CMenuList[G_NbCMenus-1].entry[G_CMenuList[G_NbCMenus-1].nb_entries-1].type=MENUITEM;
}

void AddCMenuSeparator(void)
{
  G_CMenuList[G_NbCMenus-1].nb_entries++;
  G_CMenuList[G_NbCMenus-1].entry=(TMenuEntry *) XtRealloc((char *)G_CMenuList[G_NbCMenus-1].entry, sizeof(TMenuEntry)*G_CMenuList[G_NbCMenus-1].nb_entries);
  G_CMenuList[G_NbCMenus-1].entry[G_CMenuList[G_NbCMenus-1].nb_entries-1].name=NULL;
  G_CMenuList[G_NbCMenus-1].entry[G_CMenuList[G_NbCMenus-1].nb_entries-1].command=NULL;
  G_CMenuList[G_NbCMenus-1].entry[G_CMenuList[G_NbCMenus-1].nb_entries-1].type=SEPARATOR;
}

void DestroyCMenuList(void)
{
  int i,j;
  
  for(i=0;i<G_NbCMenus;i++) {
    for(j=0;j<G_CMenuList[i].nb_entries;j++) 
      switch(G_CMenuList[i].entry[j].type) {
      case MENUITEM :
	XtFree(G_CMenuList[i].entry[j].name);
	XtFree(G_CMenuList[i].entry[j].command);
	break;
      default:
      }
    XtFree((char*) G_CMenuList[i].entry);
    XtFree(G_CMenuList[i].ident);
    XtFree(G_CMenuList[i].name);
  }
  XtFree((char *)G_CMenuList);
  G_CMenuList=NULL;
  G_NbCMenus=0;
}

/*--------------------------------------------------------------------------*/
/* file format functions */
/*-----------------------*/

void AddFileFormat(char *desc, char *pattern, char *view, char *edit, char *pixmap)
{
  int i;
  
  /* replace old format */

  for(i=0;i<G_NbFormats;i++)
    if(strcmp(G_FormatList[i].pattern, pattern)==0) {
	if(G_FormatList[i].description!=NULL) XtFree(G_FormatList[i].description);
        G_FormatList[i].description=desc;
	if(G_FormatList[i].view_command!=NULL) XtFree(G_FormatList[i].view_command);
	G_FormatList[i].view_command=view;
	if(G_FormatList[i].edit_command!=NULL) XtFree(G_FormatList[i].edit_command);
	G_FormatList[i].edit_command=edit;
	if(G_FormatList[i].pixmap_name!=NULL) XtFree(G_FormatList[i].pixmap_name);
	G_FormatList[i].pixmap_name=pixmap;
	return;
    }

  G_NbFormats++;
  G_FormatList=(TFormatList *) XtRealloc((char *)G_FormatList, sizeof(TFormatList)*G_NbFormats);
  G_FormatList[G_NbFormats-1].description=desc;
  G_FormatList[G_NbFormats-1].pattern=pattern;
  G_FormatList[G_NbFormats-1].view_command=view;
  G_FormatList[G_NbFormats-1].edit_command=edit;
  G_FormatList[G_NbFormats-1].pixmap_name=pixmap;
}

void DestroyFormatList(void)
{
  int i;
  
  for(i=0;i<G_NbFormats;i++) {
    XtFree(G_FormatList[i].description);
    XtFree(G_FormatList[i].pattern);
    XtFree(G_FormatList[i].view_command);
    XtFree(G_FormatList[i].edit_command);
    XtFree(G_FormatList[i].pixmap_name);
    /* destroy pixmap */
    if(G_FormatList[i].pixmap) XFreePixmap(XtDisplay(W_ListFile), G_FormatList[i].pixmap);
  }
  XtFree((char *)G_FormatList);
  G_FormatList=NULL;
  G_NbFormats=0;
}

void LoadFormatPixmaps(void)
{
  int i;

  for(i=0;i<G_NbFormats;i++)
    G_FormatList[i].pixmap=LoadPixmap(W_ListFile, G_FormatList[i].pixmap_name);
}

Pixmap GetFormatPixmap(char *filename)
{
  int i;

  for(i=0;i<G_NbFormats;i++) {
    if(fnmatch(G_FormatList[i].pattern, filename, 0)==0)
      return(G_FormatList[i].pixmap);
  }
  return(0);
}

int GetFormatPriority(char *filename)
{
  int i;

  for(i=0;i<G_NbFormats;i++) {
    if(fnmatch(G_FormatList[i].pattern, filename, 0) ==0)
      return i;
  }
  return(MAXINT);
}

char *GetFormatViewCommand(char *filename)
{
  int i;

  for(i=0;i<G_NbFormats;i++) {
    if(fnmatch(G_FormatList[i].pattern, filename, 0) ==0)
      return G_FormatList[i].view_command;
  }
  return(NULL);
}

char *GetFormatEditCommand(char *filename)
{
  int i;

  for(i=0;i<G_NbFormats;i++) {
    if(fnmatch(G_FormatList[i].pattern, filename, 0) ==0)
      return G_FormatList[i].edit_command;
  }
  return(NULL);
}

char *GetFormatDescription(char *filename)
{
  int i;

  for(i=0;i<G_NbFormats;i++) {
    if(fnmatch(G_FormatList[i].pattern, filename, 0) ==0)
      return G_FormatList[i].description;
  }
  return(NULL);
}

  
/*--------------------------------------------------------------------------*/
/* var list functions    */
/*-----------------------*/

void AddVarList(char *name, int int_val, Bool bool_val, char *str_val)
{
  int i;

  /* replace old definition */

  for(i=0;i<G_NbVars;i++) {
    if(strcmp(name, G_VarList[i].name)==0) {
      XtFree(G_VarList[i].name);
      G_VarList[i].name=name;
      G_VarList[i].int_val=int_val;
      G_VarList[i].bool_val=bool_val;
      if(G_VarList[i].str_val!=NULL) XtFree(G_VarList[i].str_val);
      G_VarList[i].str_val=str_val;      
      return;
    }
  }

  /* new variable */

  G_NbVars++;
  G_VarList=(TVarList *) XtRealloc((char *)G_VarList, sizeof(TVarList)*G_NbVars);
  G_VarList[G_NbVars-1].name=name;
  G_VarList[G_NbVars-1].int_val=int_val;
  G_VarList[G_NbVars-1].bool_val=bool_val;
  G_VarList[G_NbVars-1].str_val=str_val;
}

void AddPredefinedVar(char *name, int int_val, Bool bool_val, char *str_val)
{
  char *temp1;
  char *temp2=NULL;

  temp1=XtMalloc(strlen(name)+1);
  strcpy(temp1, name);
 
  if(str_val!=NULL) {
    temp2=XtMalloc(strlen(str_val)+1);
    strcpy(temp2,str_val);
  }

  AddVarList(temp1, int_val, bool_val, temp2);
}

void DestroyVarList(void)
{
  int i;
  
  for(i=0;i<G_NbVars;i++) {
    XtFree(G_VarList[i].name);
    if(G_VarList[i].str_val!=NULL) 
      XtFree(G_VarList[i].str_val);
  }
  XtFree((char *) G_VarList);
  G_VarList=NULL;
  G_NbVars=0;
}

int GetVarInt(char *name, int *result)
{
  int i;

  for(i=0;i<G_NbVars;i++)
    if(strcmp(name,G_VarList[i].name)==0) {
      *result=G_VarList[i].int_val;
      return(1);
    }

  return(0);
}

int GetVarBool(char *name, Bool *result)
{
  int i;

  for(i=0;i<G_NbVars;i++)
    if(strcmp(name,G_VarList[i].name)==0) {
      *result= G_VarList[i].bool_val;
      return(1);
    }

  return(0);
}

int GetVarString(char *name, char **result)
{
  int i;

  for(i=0;i<G_NbVars;i++)
    if(strcmp(name,G_VarList[i].name)==0) {
      *result=G_VarList[i].str_val;
      return(1);
    }

  return(0);
}

void DestroyPixmapPath(void)
{
  int i;

  for(i=0;i<G_NbPaths;i++)
    XtFree(G_PixmapPath[i]);

  XtFree((char *) G_PixmapPath);
  G_PixmapPath=NULL;
  G_NbPaths=0;
}

extern void yylex(void);
extern FILE *yyin;

void LoadConfigFile(char *rcfile)
{
  char *path;
  char *temp;
  char *token;
  int i;
  int size;

  /* default variables */
  AddPredefinedVar("IconTextMaxLength", -1, False, NULL);
  AddPredefinedVar("RefreshDirTimeInterval", 2000, False, NULL);
  AddPredefinedVar("FileIconAutoSpacing", -1, True, NULL);
  AddPredefinedVar("DebugMode", -1, True, NULL);
  AddPredefinedVar("EnableSymbolicLinks", -1, True, NULL);
  AddPredefinedVar("XtermArguments", -1, False, "/usr/bin/X11/xterm");
  AddPredefinedVar("DefaultViewCommand", -1, False, "/usr/bin/X11/xterm -e less");
  AddPredefinedVar("DefaultEditCommand", -1, False, "/usr/bin/X11/xedit");
  AddPredefinedVar("IconTextMaxLength", -1, False, NULL);
  AddPredefinedVar("WindowMenuTextMaxLength", -1, False, NULL);
  AddPredefinedVar("MaxHistoryDirs", 16, False, NULL);
  AddPredefinedVar("DndAlwaysConfirm",-1, True, NULL);
  AddPredefinedVar("NoSelectionWarning", -1, True, NULL);
  AddPredefinedVar("DndWarning", -1, True, NULL);
  AddPredefinedVar("DndDefaultCommand", -1, True, "COPY");
  AddPredefinedVar("DndCTRLCommand", -1, True, "MOVE");
  AddPredefinedVar("DndSHIFTCommand", -1, True, "LINK");
  AddPredefinedVar("WinLeftOffset", 80, True, NULL);
  AddPredefinedVar("WinTopOffset", 50, True, NULL);
  AddPredefinedVar("WinRightOffset", 80, True, NULL);
  AddPredefinedVar("WinBottomOffset", 50, True, NULL);
  AddPredefinedVar("WinXInterOffset", 10, True, NULL);
  AddPredefinedVar("WinYInterOffset", 40, True, NULL);
  AddPredefinedVar("WinXCascadeOffset", 30, True, NULL);
  AddPredefinedVar("WinYCascadeOffset", 40, True, NULL);
  AddPredefinedVar("FofDefaultCommandLine", -1, False, "fof");
  AddPredefinedVar("DefaultViewMode", -1, False, "LARGE");
  AddPredefinedVar("DefaultSortMode", -1, False, "TYPE");
  AddPredefinedVar("PixmapDefaultPath",-1, False, "/usr/include/X11/pixmaps/");

  if(access(rcfile, R_OK)==0) {
    /* load and parse config file */
    yyin = fopen(rcfile, "r");
    if(yyin==NULL) FatalError("config.c", "LoadConfigFile","Unable to open config file");
    yylex();
  } else FatalError("config.c", "LoadConfigFile", "No config file found");

  /* default pixmap path */

  if(GetVarString("PixmapDefaultPath",&path)) {
    temp = XtMalloc(strlen(path)+1);
    strcpy(temp, path);
    G_NbPaths=0;
    token=strtok(temp,":");
    while(token!=NULL) {
      G_NbPaths++;
      token=strtok(NULL,":");
    }
    G_PixmapPath=(char **) XtMalloc(sizeof(char *)*G_NbPaths);
    strcpy(temp, path);
    token = strtok(temp, ":");
    for(i=0;i<G_NbPaths;i++) {
      G_PixmapPath[i]=XtMalloc(strlen(token)+2);
      strcpy(G_PixmapPath[i], token);
      size=strlen(G_PixmapPath[i]);
      if(G_PixmapPath[i][size-1]!='/') {
	G_PixmapPath[i][size]='/';
	G_PixmapPath[i][size+1]=(char)0;
      }
      token = strtok(NULL, ":");
    }            	    
  }

  LoadFormatPixmaps();

}




